/*
 * Copyright (c) 2009 SureLogic, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.surelogic;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Target;

/**
 * Declares that regions of the object referenced by this field are to be mapped
 * into regions of the object that contains the field to which this annotation
 * is applied.
 * <p>
 * It is a modeling error if the field declaration is not also annotated with
 * {@link Unique}. It is a modeling error if the field declaration is also
 * annotated with {@link AggregateInRegion}.
 * <p>
 * Although it is possible to use this annotation to aggregate the {@code
 * Instance} region of a referenced object into a named region, it is
 * recommended that the {@link AggregateInRegion} annotation be used in this
 * case. For example, use {@code AggregateInRegion("MyRegion")} rather than the
 * equivalent {@code Aggregate("Instance into MyRegion")} which makes the
 * aggregation look similar to the more familiar {@link InRegion}.
 * 
 * <h3>Examples:</h3>
 * 
 * The below class incorporates the state referenced by the {@code observers}
 * field (i.e., the internals of the {@code HashSet} object) into the {@code
 * Instance} region of the {@code Observer} class.
 * 
 * <pre>
 * class Observer {
 *
 *   &#064;Unique
 *   &#064;Aggregate
 *   private final Set&lt;Callback&gt; observers = new HashSet&lt;Callback&gt;()
 *   ...
 * }
 * </pre>
 * 
 * We can change the above example to use a named region instead of the default
 * {@code Instance} region. The below class defines a region, called {@code
 * ObserverRegion}, that includes the integer {@code notifyCounter} and the
 * state referenced by the {@code observers} field (i.e., the internals of the
 * {@code HashSet} object).
 * 
 * <pre>
 * &#064;Region(&quot;private ObserverRegion&quot;)
 * class Observer {
 *
 *   &#064;InRegion(&quot;ObserverRegion&quot;)
 *   private int notifyCounter = 0;
 *
 *   &#064;Unique
 *   &#064;Aggregate(&quot;Instance into ObserverRegion&quot;)
 *   private final Set&lt;Callback&gt; observers = new HashSet&lt;Callback&gt;()
 *   ...
 * }
 * </pre>
 * 
 * However, the above syntax is <em>not recommended</em>. Instead it is
 * recommended to use the {@link AggregateInRegion} annotation as shown below.
 * 
 * <pre>
 * &#064;Region(&quot;private ObserverRegion&quot;)
 * class Observer {
 *
 *   &#064;InRegion(&quot;ObserverRegion&quot;)
 *   private int notifyCounter = 0;
 *
 *   &#064;Unique
 *   &#064;AggregateInRegion(&quot;ObserverRegion&quot;)
 *   private final Set&lt;Callback&gt; observers = new HashSet&lt;Callback&gt;()
 *   ...
 * }
 * </pre>
 * 
 * The syntax if only a portion of a referenced object is to be aggregated is
 * more complex. Consider the highly contrived example below.
 * 
 * <pre>
 * &#064;Regions({
 *   &#064;Region(&quot;private OrderRegion&quot;),
 *   &#064;Region(&quot;private InvRegion&quot;) })
 * &#064;RegionLocks({
 *   &#064;RegionLock(&quot;ShopLock is this protects OrderRegion&quot;),
 *   &#064;RegionLock(&quot;InvLock  is inv  protects InvRegion&quot;) })
 * public class Shop {
 *
 *   &#064;InRegion(&quot;OrderRegion&quot;)
 *   private int ordersPlaced;
 * 
 *   private static class SandwichInventory {
 *     int bread;
 *     int mustard;
 *     int ordersPlaced;
 *   }
 * 
 *   &#064;Unique
 *   &#064;Aggregate(&quot;bread into InvRegion, mustard into InvRegion, ordersPlaced into OrderRegion, Instance into Instance&quot;)
 *   private final SandwichInventory inv = new SandwichInventory();
 *
 *   void order() {
 *     synchronized (inv) {
 *       inv.bread++;
 *       inv.mustard++;
 *     }
 *     synchronized (this) {
 *       ordersPlaced++;
 *       inv.ordersPlaced++;
 *     }
 *   }
 *   ...
 * }
 * </pre>
 * 
 * In the above example the fields that count the number of orders placed are
 * protected by synchronizing on {@code this} while the inventory fields (
 * {@code bread} and {@code mustard}) are protected by synchronizing on {@code
 * inv}. All the state of the {@code SandwichInventory} must be placed into
 * named regions (even if you never use one of the named regions, e.g., you
 * create a <em>JunkRegion</em>) and the aggregation must be completed with a
 * mapping of the region {@code Instance} of the referenced object, usually
 * {@code Instance into Instance}.
 * <p>
 * Models of the sort shown for the {@code Shop} class above are discouraged and
 * should occur rarely in Java code that follows object-oriented design
 * principles.
 * 
 * <h3>Javadoc usage notes:</h3>
 * 
 * This annotation may placed in Javadoc, which can be useful for Java 1.4 code
 * which does not include language support for annotations, via the
 * <code>&#064;annotate</code> tag.
 * 
 * <pre>
 * class Observer {
 *
 *   /**
 *    * &#064;annotate Unique
 *    * &#064;annotate Aggregate
 *    *&#047;
 *   private final Set&lt;Callback&gt; observers = new HashSet&lt;Callback&gt;()
 *   ...
 * }
 * </pre>
 * 
 * @see AggregateInRegion
 * @see Unique
 */
@Documented
@Target(ElementType.FIELD)
public @interface Aggregate {
	/**
	 * The value of this attribute must conform to the following grammar (in <a
	 * href="http://www.ietf.org/rfc/rfc4234.txt">Augmented Backus&ndash;Naur
	 * Form</a>):
	 * 
	 * <pre>
	 * value = regionMapping *(&quot;,&quot; regionMapping)
	 * 
	 * regionMapping = simpleRegionSpecification &quot;into&quot; regionSpecification
	 * 
	 * regionSpecification = simpleRegionSpecificaion / qualifiedRegionName
	 * 
	 * simpleRegionSpecification = IDENTIFIER / &quot;[&quot; &quot;]&quot;               ; Region of the class being annotated
	 * 
	 * qualifedRegionName = IDENTIFIER *(&quot;.&quot; IDENTIFIER) : IDENTIFER  ; Static region from the named, optionally qualified, class
	 * 
	 * IDENTIFIER = Legal Java Identifier
	 * </pre>
	 * 
	 * <p>
	 * In {@code A into B}, the first RegionSpecification is relative to the
	 * object referenced by the field; the second is relative to the object that
	 * contains the field.
	 */
	String value() default "Instance into Instance";
}
